// ErrorLogger.cpp

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "ErrorLogger.h"
#include "Exceptions.h"
#include <fstream>
#include <iostream>
#include <string>
#include <list>

using std::cout;
using std::ofstream;
using std::string;
using std::list;

ErrorLogger* ErrorLogger::instance = 0;

ErrorLogger* ErrorLogger::Instance()
{
	if (!instance)
		instance = new ErrorLogger;
	return instance;
}

ErrorLogger::ErrorLogger()
{}

void ErrorLogger::server_said(const string& quote)
{
	string temp("***SERVER***\n");
	temp += quote;
	
	errLog.push_back( temp );
	if (errLog.size() > 10)
		errLog.pop_front();
}

void ErrorLogger::client_said(const string& quote)
{
	string temp("***CLIENT***\n");
	temp += quote;
	
	errLog.push_back( temp );
	if (errLog.size() > 10)
		errLog.pop_front();
}

void ErrorLogger::dump_to_file() 
throw (exc::FileAccess)
{
	ofstream ofs("errlog.txt", ios::out);
	if (!ofs)
	{
		throw exc::FileAccess("ErrorLogger::dump_to_file()", "errlog.txt");
	}
	else
	{
		list<string>::iterator iter = errLog.begin();
		while ( iter != errLog.end() )
		{
			ofs << *iter << endl;
			iter++;
		}
	}
	ofs.close();

	cout
	 	<< "A log of the application error has been written to the working" << endl
		<< "directory (errlog.txt).  Please send it, along with a description" << endl
		<< "of what you were doing at the time of the error (e.g. click anything?)" << endl
		<< "to DrDave@PenguinPowered.com -- thanks for helping out!" << endl << endl;
}