// Exceptions.h
//
// Defines the exception objects that are thrown by objects in PicMonger...

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _EXCEPTIONS_H
#define _EXCEPTIONS_H

#include <string>


using std::string;
class HtmlGenerator;
namespace exc {

class Basic {
	public:
		Basic( 	const string& location = "Unknown",
				const string& condition = "Unknown"	);
		virtual ~Basic();

		void handle();
		
	protected:
		bool   abort;
		string errmsg;
		string loc;
		string cond;
		virtual void exc_specific();
};

class Fatal : public Basic {
	public:
		Fatal(	const string& location = "Unknown",
				const string& condition = "Fatal Error" );
		virtual ~Fatal();
		
	protected:
		virtual void exc_specific();	
};

class NullPointer : public Fatal
{
	public:
		NullPointer( const string& location = "Unknown" );
		virtual ~NullPointer();

	protected:
		void exc_specific();
};

class FileAccess : public Basic
{
	public:
		FileAccess( const string& location = "Unknown",
					const string& filename = "Filename Not Specified" );
		virtual ~FileAccess();

	protected:
		void exc_specific();

	private:
		string fname;
}; 

class ConnectFail : public Basic
{
	public:
		ConnectFail( 	const string& location,
						const char *dotquad,
						int port_num	);

	protected:
		void exc_specific();
		
	private:
		string dquad;
		int port;
};

class ResolveFail : public Basic
{
	public:
		ResolveFail( 	const string& location,
						const string& address	);

	protected:
		void exc_specific();
		
	private:
		string addr;
};

class ReadFail : public Basic
{
	public:
		ReadFail( const string& location );
};

class WriteFail : public Basic
{
	public:
		WriteFail( const string& location );
};

class NNTP_Error : public Basic
{
	public: 
		NNTP_Error( const string& location, const string& errorline );

	protected:
		void exc_specific();

	private:
		string err;
};

class TimeOut : public Basic
{
	public:
		TimeOut( 	const string& location, HtmlGenerator *htgen,
					const string& pending_io = "(Err During Read)"	);
	protected:
		void exc_specific();

	private:
		string pend_io;
		HtmlGenerator *p_htgen;
};

//// Signals--->

class ScannerShutdown 
{
	public:
		ScannerShutdown()
			{  }
};



// end of exc namespace
};
#endif