// FileTypeState.cpp

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "FileTypeState.h"
#include <fstream>
#include <stdlib.h>	// need getenv()

// Implemented as a Singleton to simplify access from both filetype dialog and
// newsscanner without having to maintain references to each other

FileTypeState *FileTypeState::instance = 0;
FileTypeState *FileTypeState::Instance()
{
	if (!instance)
		instance = new FileTypeState;
	return instance;
}

FileTypeState::FileTypeState()
{
	statefilename = getenv("HOME");
	statefilename += "/.picmonger/ftstate.dat";

	if (!restore_state())
		initialize();
}

FileTypeState::~FileTypeState()
{}

const bool FileTypeState::read_state_of( FileTypes type ) const
{
	if ( static_cast<unsigned int>(type) >= settings.size() ) return false;
	if ( type < 0 ) return false;
	
	return settings[ type ];
}

void FileTypeState::set_state_of( FileTypes type, bool state )
{
	if ( static_cast<unsigned int>(type) >= settings.size() ) return;
	if ( type < 0 ) return;

	settings[ type ] = state;
	save_state();
}

void FileTypeState::save_state() throw (exc::FileAccess)
{
	ofstream ofs( statefilename.c_str(), ios::out );
	if (!ofs)
		throw exc::FileAccess("FileTypeState::save_state()", statefilename);

	for (int i=0; i<FILETYPE_COUNT; i++)
	{
		ofs << settings[i] << " " << flush;
	}

	ofs.close();
}

void FileTypeState::initialize()
// this is called if no statefile is found, or statefile is empty.
// filetype_count is the total # of filetypes recognized by PicMonger
// (those are enumerated in NewsScanner.h)
{
	settings.clear();
	for (int i=0 ; i < FILETYPE_COUNT ; i++)
	{
		if ( i == FT_JPG )			// default is only JPEG type selected
			settings.push_back( true );
		else
			settings.push_back( false );
	}
	save_state();
}
	
bool FileTypeState::restore_state()
// returns # of boolean values read from the statefile
{
	ifstream ifs( statefilename.c_str(), ios::in );
	if (!ifs) return false;		// tells the caller we need to be initialized first

	bool tempstate;
	for (int i=0; i < FILETYPE_COUNT ; i++)
	{
		ifs >> tempstate;
		settings.push_back( tempstate );
	}

	return true;
}

	
