// FilenameCleaner.cpp
// 

#include "FilenameCleaner.h"
#include "FileTypeState.h" // need enum FileTypes
#include <ctype.h>	// need tolower()
#include <strstream>

using std::strstream;

FilenameCleaner::FilenameCleaner()
{
	p_ftState = FileTypeState::Instance();
}

FilenameCleaner::~FilenameCleaner()
{}

bool FilenameCleaner::validate_filename( string &fn )
{
	int bsl_pos = fn.find_last_of("\\");
	int fsl_pos = fn.find_last_of("/");

	// set fn_idx to 1st char after the last / or \ in the filename, if any --
	// note that if neither are found, ++bsl_pos and ++fsl_pos both evaluate to 0.
	int fn_idx = ( bsl_pos > fsl_pos ) ? ++bsl_pos : ++fsl_pos;

	// strip off the path (everything up to the last \ or / character, inclusive)
	fn = fn.substr( fn_idx, fn.size()-fn_idx );	

	// convert what's left to lowercase only 
	// and convert problematic chars to '_' chars
	char *temp_str = new char[fn.size() + 1];
	fn.copy( temp_str, fn.size() );
	temp_str[ fn.size() ] = '\0';

	char *c = temp_str;
	while (*c) 
	{ 
		*c = tolower(*c); 
		
		if ( 	(*c == ' ') || 
			(*c == '?') || 
			(*c == '!') || 
			(*c == '-') || 
			(*c == '$') ||
			(*c == '#')	||
			(*c == '(') ||
			(*c == ')') ||
			(*c == '\'') ) 
		{ *c = '_'; }

 		c++; 
	}
	
	fn = temp_str;
	delete [] temp_str;

	// check for a wanted filename extension and return
	int r;
	if (p_ftState->read_state_of(FT_JPG) && (	((r = fn.find(".jpg")) > 0) 
                                  		 || ((r = fn.find(".jpeg")) > 0) 	))
		{ return true; }
	else if ( ((r = fn.find(".gif")) > 0) && (p_ftState->read_state_of(FT_GIF)) ) 	
		{ return true; }
	else if ( ((r = fn.find(".mpg")) > 0) && (p_ftState->read_state_of(FT_MPG)) ) 	
		{ return true; }
	else if ( ((r = fn.find(".mov")) > 0) && (p_ftState->read_state_of(FT_MOV)) ) 	
		{ return true; }
	else if ( ((r = fn.find(".avi")) > 0) && (p_ftState->read_state_of(FT_AVI)) ) 	
		{ return true; }
	else if ( ((r = fn.find(".mp3")) > 0) && (p_ftState->read_state_of(FT_MP3)) ) 	
		{ return true; }
	else if ( ((r = fn.find(".csv")) > 0) && (p_ftState->read_state_of(FT_CSV)) ) 	
		{ return true; }
	else return false;
}

string FilenameCleaner::get_UUE_filename()
{
	strstream instr;	// create a stream from the current data buffer
	instr << inbuf;
	
	string beginline;
	getline( instr, beginline, '\n' ); 	// copy the first line of the stream to
										// string beginline

	int found = beginline.find("begin"); 	// check to see if it IS the begin line
	while ( (found < 0) && (instr.good()) )	// and loop until it's found
	{
		getline( instr, beginline, '\n' );
		found = beginline.find("begin");
	}

	// if found < 0 on exit, we never found the begin line, so return empty string
	if (found < 0) return "";
	
	int idx = beginline.find(" ");			// position of space p "begin"
	idx = beginline.find(" ", ++idx);		// position of space p file perms
	idx++;									// position of beginning of filename

	// return everything from the beginning of the filename to end of this line	
	return (beginline.substr( idx, beginline.size() - idx ));
}

string FilenameCleaner::get_MIME_filename()
{
	strstream instr;
	instr << inbuf;

	string headline;
	getline( instr, headline, '\n' );
	int found_lower = headline.find("filename");
	int found_upper = headline.find("FILENAME");

	while ( (found_lower < 0) && (found_upper < 0) && (instr.good()) )
	{
		getline( instr, headline, '\n' );
		found_lower = headline.find("filename");
		found_upper = headline.find("FILENAME");
	}

	// if both found_lower and found_upper are <0, we never found the header line
	// so print a message and return an empty string
	if ( (found_lower < 0) && (found_upper < 0) )
	{
		cerr << "Couldn't find Content-Disposition header!" << endl;
		return "";
	}
	
	int fn_start = headline.find("\"");
	int fn_end = headline.rfind("\"");

	// check if we found both quotes enclosing the filename (return "" if not)
	if ( (fn_start < 0) || (fn_end < 0) || (fn_start == fn_end) )
	{
		return "";
	}
	
	fn_start++;
	
	int fn_len = fn_end - fn_start;
	if (fn_len > 0)
	{
		string fname = headline.substr(fn_start, fn_end-fn_start);
		return fname;
	}
	else
	{
		return ("");
	}	
}

string FilenameCleaner::fix_UUE_buffer( const string& old, const string& fixed )
{
	replace_UUE_filename( old, fixed );
	fix_newlines( fixed );
	return inbuf;
}

string FilenameCleaner::fix_MIME_buffer( const string& old, const string& fixed )
{
	replace_MIME_filename( old, fixed );
	fix_newlines( fixed );
	return inbuf;
}

void FilenameCleaner::replace_UUE_filename( const string& old, const string& fixed )
{
	int begin_start = inbuf.find("begin ");
		
	if (begin_start >= 0)
	{
		int old_start = inbuf.find( old, begin_start );	
		inbuf.replace( old_start, old.size(), fixed );
	}
}

void FilenameCleaner::replace_MIME_filename( const string& old, const string& fixed )
{
	int fn_line_start = inbuf.find("filename=");
	if (fn_line_start < 0)
		fn_line_start = inbuf.find("FILENAME=");
		
	if (fn_line_start >= 0)
	{
		int old_start = inbuf.find( old, fn_line_start );
		inbuf.replace( old_start, old.size(), fixed );
	}
}

void FilenameCleaner::fix_newlines( const string& fn )
// removes the trailing '\n' from filename references, if any
{
	int idx = 0;
	while ( (idx = inbuf.find( fn, idx )) >= 0)
	{
		int nl_idx = inbuf.find( "\r", idx );
		if (nl_idx == -1) return;
		
		//inbuf.remove( nl_idx, 1 );	// not implemented right in GNU C++
		inbuf.replace( nl_idx, 1, "" );	// workaround
		idx = (nl_idx < static_cast<int>(inbuf.size())) ? nl_idx : inbuf.size()-1;
	}
}

