// GroupState.cpp
// 
//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "GroupState.h"
#include "Exceptions.h"
#include <fstream>
#include <stdio.h>	// need sscanf
#include <stdlib.h>	// need getenv

using std::string;

GroupState::GroupState(const string& groupname, const string& max_msg_id)
	: group(groupname)
{
	sscanf( max_msg_id.c_str(), "%d", &max_msg );
	
	fname = getenv("HOME");
	fname += "/.picmonger/groupstate.dat";
	
	read_state();
}

void GroupState::set_last_message(const string& msg_id)
{
	GroupStateData::iterator iter = state.find( group );
	if ( iter == state.end() )
	{
		state.insert( StatePair( group, msg_id ) );
	}
	else
		state[ group ] = msg_id;

	save_state();
}

string GroupState::get_last_message()
{
	int last_int;
	string lastmsg = state[ group ];// inserts group automatically if not there
	
	if (lastmsg == "") return "";	// return empty if we have no record for this grp
	
	sscanf( lastmsg.c_str(), "%d", &last_int );

	if ( last_int > max_msg )		// msg #s have rolled over in this group
		return "";					// so return as if we've never been before
	else
		return lastmsg;
}

void GroupState::read_state()
{
	ifstream ifs( fname.c_str(), ios::in );
	string key, value;

	if (!ifs) 
		return;		// quietly leave the map empty if there's no statefile

	while (ifs.good())
	{
		ifs >> key;
		ifs >> value;
		state.insert( StatePair( key, value ) );
	}
	
	ifs.close();
}	

void GroupState::save_state() throw (exc::FileAccess)
{
	ofstream ofs( fname.c_str(), ios::out );

	if (!ofs)
		throw exc::FileAccess("GroupState::save_state()", fname );
	else 
	{
		GroupStateData::iterator iter = state.begin();
		while (iter != state.end())
		{
			ofs << iter->first << " " << iter->second << " ";
			iter++;
		}
		ofs.close();
	}
}
