// HtmlGenerator.h
//
// An object of class HtmlGenerator is responsible for generating the HTML for preview pages.

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _HTML_GENERATOR_H
#define _HTML_GENERATOR_H

using namespace std;

#include <string>
#include <list>
#include <fstream>
#include "Exceptions.h"
#include "PreviewPage.h"

class HtmlGenerator 
{
	public:
		HtmlGenerator(const string& path);
		~HtmlGenerator();

		void add(const string& datafile);
		void finish();

		void set_logopath( const string& logo_path );
		void set_bgcolor( const string& bg );
		void set_groupname( const string& group );
		void set_basename( const string& bname );
						
	private:
		PreviewPage* p_current;
		string currentFile;
		
		PreviewPage* p_last;
		string lastFile;
		
		string htmlpath;
		
		string logopath;				// the logo to put on top of every page
		string bgcolor;					// color for the page background

		string groupname;				// name of newsgroup previewed 
		string basename;				// base name for generated html files

		uint t_height, t_width;			// thumbnail dimension constraints
		int min_height, min_width;		// min h and w for keeping images

		uint img_per_row;				// # of images to put in each row
		uint row_per_page;				// # of rows each table

		list<PreviewPage *> pages;		// keeps a running list of created preview pp
		list<string> files;				// list of files to add to pages

		void clear_pages();
		void clear_files();
		
		bool sortOutput;
		bool showLogo;
		bool showGroup;
		bool showFilenames;

		int htmlcount;
		string html_num();
};

#endif // HtmlGenerator not already included

