// HtmlOptionState.cpp
// 
//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "HtmlOptionState.h"
#include <fstream>

using std::ifstream;
using std::ofstream;

HtmlOptionState* HtmlOptionState::instance = 0;
HtmlOptionState* HtmlOptionState::Instance()
{
	if (!instance)
		instance = new HtmlOptionState;
	return instance;
}

HtmlOptionState::HtmlOptionState()
{
	statefile = getenv("HOME");
	statefile += "/.picmonger/htoptstate.dat";

	if (!restore_state()) initialize(); 	// init to defaults if state can't be restored
}

HtmlOptionState::~HtmlOptionState()
{}

void HtmlOptionState::save_state() throw (exc::FileAccess)
{
	ofstream ofs( statefile.c_str(), ios::out );
	if (!ofs) throw exc::FileAccess("HtmlOptionState::save_state()", statefile );

	ofs << rows << " " << cols << " " << t_height << " " << t_width << " "
		<< showBanner << " " << showGroup << " " << showFilename << " " << sortFiles 
		<< " ";
}

bool HtmlOptionState::restore_state()
{
	ifstream ifs( statefile.c_str(), ios::in );
	if (!ifs) return false;

	ifs >> rows;
	ifs >> cols;
	ifs >> t_height;
	ifs >> t_width;
	ifs >> showBanner;
	ifs >> showGroup;
	ifs >> showFilename;
	ifs >> sortFiles;
	

	return true;
}

void HtmlOptionState::initialize()
{
	const uint ROW_DEF = 4;
	const uint COL_DEF = 5;
	const uint THUMB_DIM_DEF = 150;
	
	rows = ROW_DEF;
	cols = COL_DEF;
	t_height = t_width = THUMB_DIM_DEF;
	showBanner = showGroup = showFilename = sortFiles = true;
	useAuth = false;

	try {	
		save_state();
	} catch(exc::Basic& e) {
		e.handle();
		exit(1);
	}
}