// HtmlOptionState.h
//
// Implemented as a Singleton for easy access from class HtmlOptions, class 
// HtmlGenerator and class PreviewPage, HtmlOptionState is reponsible for 
// saving/restoring HtmlOptions settings between sessions.

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _HTMLOPTIONSTATE_H
#define _HTMLOPTIONSTATE_H

#include "Exceptions.h"
#include <string>
using std::string;

class HtmlOptionState {
	public:
		static HtmlOptionState* Instance();
		~HtmlOptionState();

		// GET methods
		inline const uint get_rows() const	{ return rows; }
		inline const uint get_cols() const  { return cols; }
		inline const uint get_t_height() const { return t_height; }
		inline const uint get_t_width() const  { return t_width; }
		inline const bool get_show_banner() const { return showBanner; }
		inline const bool get_show_groupname() const { return showGroup; }
		inline const bool get_show_filenames() const { return showFilename; }
		inline const bool get_sort_files() const { return sortFiles; }
		

		// SET methods
		inline void set_rows( uint r )		{ rows = r; }
		inline void set_cols( uint c )		{ cols = c; }
		inline void set_t_height( uint h )	{ t_height = h; }
		inline void set_t_width( uint w )	{ t_width = w; }
		inline void set_show_banner( bool show )	{ showBanner = show; }
		inline void set_show_groupname( bool show )	{ showGroup = show; }
		inline void set_show_filenames( bool show ) { showFilename = show; }
		inline void set_sort_files( bool sortem ) { sortFiles = sortem; }
		
		
		// FILE operations
		void save_state() throw (exc::FileAccess);
		bool restore_state();
		
	protected:
		static HtmlOptionState* instance;
		HtmlOptionState();
	
	private:
		string statefile;
		uint rows, cols, t_height, t_width;
		bool showBanner, showGroup, showFilename, sortFiles, useAuth;

		void initialize();

};

#endif
