// HtmlOptions.cpp
// 
//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "HtmlOptions.h"
#include "HtmlOptionState.h"
#include "Exceptions.h"

// Singleton implementation for easy reference by HtmlGenerator when needed
// (Doesn't need to be anymore because of addition of HtmlOptionState class -- consider
// reverting to a non-Singleton implementation)

HtmlOptions* HtmlOptions::_instance = 0;
HtmlOptions* HtmlOptions::Instance()
{
	if (!_instance)
		_instance = new HtmlOptions;
	return _instance;
}

HtmlOptions::HtmlOptions()
	: Gtk::Frame("HTML Generator Options")
{
	p_htOpts = HtmlOptionState::Instance();
	
	set_border_width( 5 );
	set_label_align( 0.5 );
	
	tblMain = manage( new Gtk::Table( 7,4 ) );	// rows, columns

	// create checkbuttons and initialize their values
	ckbShowBanner = manage( new Gtk::CheckButton("Show Banner at Top") );
	ckbShowBanner->set_active( p_htOpts->get_show_banner() );
	
	ckbShowGroupname = manage (new Gtk::CheckButton("Show Group Name at Top") );
	ckbShowGroupname->set_active( p_htOpts->get_show_groupname() );

	ckbShowFilenames = manage (new Gtk::CheckButton("Show Filenames") );
	ckbShowFilenames->set_active( p_htOpts->get_show_filenames() );

	ckbSortFiles = manage (new Gtk::CheckButton("Sort Decoded Files") );
	ckbSortFiles->set_active( p_htOpts->get_sort_files() );

	// define allowed ranges for thumbnail dimensions and # of rows or columns
	const uint MIN_THUMB_DIM = 50;
	const uint MAX_THUMB_DIM = 300;
	const uint MIN_ROWS_COLS = 1;
	const uint MAX_ROWS_COLS = 20;

	// thumbnail dimension spinbuttons and frame

	// create adjustments for max-dimension spinbuttons 
	adjHeight = manage( 
		new Gtk::Adjustment( p_htOpts->get_t_height(), MIN_THUMB_DIM, MAX_THUMB_DIM ) );
	adjWidth  = manage( 
		new Gtk::Adjustment( p_htOpts->get_t_width(), MIN_THUMB_DIM, MAX_THUMB_DIM ) );

	frmThumbs = manage( new Gtk::Frame("Thumbnails"));
	frmThumbs->set_label_align( 0.5 );
	frmThumbs->set_shadow_type( GTK_SHADOW_OUT );

	tblThumbs = manage( new Gtk::Table(2,2) );
	
	lblHeight = manage( new Gtk::Label("Height Limit:") );
	lblWidth  = manage( new Gtk::Label("Width Limit:") );

	spinHeight = manage( new Gtk::SpinButton( *adjHeight ) );
	spinHeight->set_update_policy( GTK_UPDATE_IF_VALID );

	spinWidth  = manage( new Gtk::SpinButton( *adjWidth ) );
	spinWidth->set_update_policy( GTK_UPDATE_IF_VALID );

	tblThumbs->attach( *lblHeight, 0,1, 0,1 );
	tblThumbs->attach( *lblWidth, 0,1, 1,2 );
	tblThumbs->attach( *spinHeight, 1,2, 0,1 );
	tblThumbs->attach( *spinWidth, 1,2, 1,2 );
	tblThumbs->show_all();

	frmThumbs->add( *tblThumbs );
	frmThumbs->show_all();
	
	// # rows/columns spinbuttons
	adjRows = manage( 
		new Gtk::Adjustment( p_htOpts->get_rows(), MIN_ROWS_COLS, MAX_ROWS_COLS ) );
	adjCols = manage( 
		new Gtk::Adjustment( p_htOpts->get_cols(), MIN_ROWS_COLS, MAX_ROWS_COLS ) );
	lblRows = manage( new Gtk::Label("# of Rows:") );
	lblCols = manage( new Gtk::Label("# of Cols:") );
	spinRows = manage( new Gtk::SpinButton( *adjRows ) );
	spinRows->set_update_policy( GTK_UPDATE_IF_VALID );
	spinCols = manage( new Gtk::SpinButton( *adjCols ) );
	spinCols->set_update_policy( GTK_UPDATE_IF_VALID );
	
	// ......... Now pack everything we've created ...........
	add ( *tblMain );
	// checkbuttons for boolean values
	tblMain->attach( *ckbShowBanner, 1,3 , 0,1 );
	tblMain->attach( *ckbShowGroupname, 1,3 , 1,2 );
	tblMain->attach( *ckbShowFilenames, 1,3 , 2,3 );
	tblMain->attach( *ckbSortFiles, 1,3 , 3,4 );
	
	// spinbuttons for numeric values
	tblMain->attach( *frmThumbs, 2,4, 4,6 );
	
	tblMain->attach( *lblRows, 0,1 , 4,5 );
	tblMain->attach( *spinRows, 1,2 , 4,5 );
	tblMain->attach( *lblCols, 0,1 , 5,6 );
	tblMain->attach( *spinCols, 1,2 , 5,6 );

	btnSetOpts = manage( new Gtk::Button("Set HTML Options") );
	btnSetOpts->clicked.connect( slot( this, &HtmlOptions::cbSetOpts ) );
	tblMain->attach( *btnSetOpts, 0,4 , 6,7 );
	
	show_all();
}

HtmlOptions::~HtmlOptions()
{}

void HtmlOptions::cbSetOpts()
{
	p_htOpts->set_rows( static_cast<uint>(spinRows->get_value_as_int()) );	
	p_htOpts->set_cols( static_cast<uint>(spinCols->get_value_as_int()) );
	p_htOpts->set_t_height( static_cast<uint>(spinHeight->get_value_as_int()) );	
	p_htOpts->set_t_width( static_cast<uint>(spinWidth->get_value_as_int()) );	
	p_htOpts->set_show_banner( ckbShowBanner->get_active() );	
	p_htOpts->set_show_groupname( ckbShowGroupname->get_active() );	
	p_htOpts->set_show_filenames( ckbShowFilenames->get_active() ); 
	p_htOpts->set_sort_files( ckbSortFiles->get_active() );
	
	try {
		p_htOpts->save_state();
	} catch (exc::Basic& e) {
		e.handle();
	} 
}