// HtmlPage.cpp
// Implements the base class for HTML pages generated by HtmlGenerator

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "HtmlPage.h"
#include "Exceptions.h"

#ifndef PKGDATADIR
	#define PKGDATADIR "/usr/local/share/PicMonger"
#endif

using std::string;

HtmlPage::HtmlPage(	const string& path, 
					const string& _filename,
					const string& group)
	: htmlpath( path ), filename( _filename ), bgColor("#000066"), bgImage(""),
	bannerPath("file:" PKGDATADIR "/pmlogo_big.jpg"),
	groupName( group ), pageNumber("?")
{}

HtmlPage::~HtmlPage()
{}

void HtmlPage::write() throw (exc::FileAccess)
{
	// add terminating slash to html filepath if not already there
	if (htmlpath[ htmlpath.size()-1 ] != '/')
		htmlpath += '/';

	string outfile = htmlpath + filename;

	ofstream ofs( outfile.c_str(), ios::out );
	if (!ofs)
		throw exc::FileAccess("HtmlPage::write()", outfile);
	else
	{
		write_header(ofs);
		write_content(ofs);
		write_footer(ofs);
		ofs.close();
	}
}

void HtmlPage::write_header(ofstream &fout)
{
	string caption;
	string title;
	
	if (showPageNumber)
	{
		caption = "Image Previews: Page ";
		caption += pageNumber;

		title = "PicMonger ";
		title += caption;
	}
	else
	{
		caption = "Image Previews";
		title = "Picmonger Image Previews";
	}

	fout << "<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">" << endl;
	fout << "<html>" << endl;
	fout << "<head>" << endl;
	fout << "\t<title>" << title << "</title>" << endl;
	fout << "\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">" << endl;
	fout << "\t<meta name=\"GENERATOR\" content=\"PicMonger by David J.V. Brown\">" << endl;
	fout << "</head>" << endl;
	
	// write body line and colors...
	fout << "<body bgcolor=\"" << bgColor << "\" " ;
	if (bgImage != "")
		fout << "background=\"" << bgImage << "\" ";
	fout << "text=\"#FFFFFF\" link=\"#3333FF\" vlink=\"#AAAAAA\" >" << endl << endl;
	
	if (showBanner)
	{
		fout << "<center>" << endl;
		fout << "<img src=\"" << bannerPath << "\" border=0 >" << endl;
		fout << "</center>" << endl;
		fout << "<hr width=\"90%\">" << endl;
	}
	
	fout << endl;
	if (showGroupName)
	{
		fout << "<center><h2>" << groupName << "</h2></center>" << endl;
	}
	fout << "<center><h3>" << caption << "</h3></center>" << endl;
	fout << "<p>" << endl;
}


void HtmlPage::write_footer(ofstream &fout)
{
	// close remaining open tags from header
	fout << "</body>" << endl;
	fout << "</html>" << endl;			 
}










