// HtmlPage.h
//
// This is the base class for all HTML pages created by the HTML generator;

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _HTMLPAGE_H
#define _HTMLPAGE_H

#include <string>
#include <fstream>
#include "Exceptions.h"

using std::string;

class HtmlPage {

	public:
		HtmlPage(	const string& path, 
					const string& _filename,
					const string& group);
		virtual ~HtmlPage();
		
		void write() throw (exc::FileAccess);
		
		inline void set_show_banner(bool show)	{ showBanner = show; }
		inline void set_show_pagenum(bool show)	{ showPageNumber = show; }
		inline void set_show_group(bool show)	{ showGroupName = show; }
		inline void set_pagenum(const string& pnum)	{ pageNumber = pnum; }
		inline void set_bg_image(const string& bgimg) 	{ bgImage = bgimg; }
		inline void set_bg_color(const string& bgcol)	{ bgColor = bgcol; }
		inline void set_banner_path(const string& bp)	{ bannerPath = bp; }

	protected:
		string htmlpath;
		string filename;

		string bgColor;
		string bgImage;
		string bannerPath;
		string groupName;
		string pageNumber;

		bool showBanner;
		bool showPageNumber;
		bool showGroupName;
		
		virtual void write_header(ofstream& fout);
		virtual void write_content(ofstream& fout) = 0;
		virtual void write_footer(ofstream& fout);
		
};


#endif