//	InetSocket.h
//
//	Base Class for C++ PF_INET sockets
//

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef _INETSOCKET_H
#define _INETSOCKET_H

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "Exceptions.h"

class InetSocket
{
	public:
		InetSocket(int type=SOCK_STREAM, int protocol=0);

		// for client sockets
		int s_connect ( const char* dotquad, int port ) throw (exc::ConnectFail);

		// for server sockets
		int s_bind( int port, const char *dotquad = (const char *)0 );
		int s_listen( int queue_len );
		int s_accept();

		// for both client and server sockets
		int s_read( int read_fd, void *data, int read_max ) throw (exc::ReadFail);
		int s_write( const void *data, int write_max ) throw (exc::WriteFail);
		int s_close();

		inline const int get_fd() const { return local_fd; } 
		
	private:
		int type_;
		int protocol_;

		int local_fd; 
		sockaddr_in local_addr;
		socklen_t local_addr_len;
		
		int remote_fd;
		sockaddr_in remote_addr;
		socklen_t remote_addr_len;

		
};

#endif
