// MongerList.cpp
// 
// Implements a an object that manages lists of newsgroup names or server URLs to
// be used by the PicMonger GUI in a dropdown (combo) box.

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "MongerList.h"
#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <unistd.h>

using namespace std;

MongerList::MongerList( const string &filename )
	: iter( item_list.begin() ), size (0), fname( filename )
{
	read_file();
}

MongerList::~MongerList()
{
	// nothing to clean up
}

void MongerList::add_item(const string &item)
{
	item_list.push_back( item );
	size = item_list.size();
	unlink( fname.c_str() );
	write_file();
}

void MongerList::remove_item(const string &item)
{
	if (!item_list.empty())
	{
		remove(item_list.begin(), item_list.end(), item);	// from standard algorithms
		item_list.pop_back();
		size = item_list.size();
		unlink( fname.c_str() );
		write_file();
	}
}

const int MongerList::length() const	{ return item_list.size(); }

const string MongerList::item()
{
	static string nitem = " ";
	
	nitem = *iter;
	
	return nitem;
}

bool MongerList::next()		// returns false when you run off the end of the list
{
	if (iter != item_list.end()) 
		iter++;
	else
		return false;

	return (iter != item_list.end());
}

void MongerList::reset()
{
	iter = item_list.begin();
}

bool MongerList::read_file()
{
	item_list.clear();
	fstream fin;
	fin.open( fname.c_str(), ios::in );
	if (!fin.is_open()) return false;

	string item = "";

	do	{
		fin >> item;
		if (item != "") item_list.push_back(item);
		item = "";
		} 	
	while (!fin.eof());
	
	fin.close();

	size = item_list.size();
	return true;
}

bool MongerList::write_file()
{
	fstream fout;
	fout.open(fname.c_str(), ios::out);
	if (!fout.is_open()) return false;

	iter = item_list.begin();
	while (iter != item_list.end())
	{
		fout << (*iter) << endl;
		iter++;
	}
	fout.close();
	return true;
}

	