// NewsScannerView.cpp
// 
// A simple viewer for the status of a NewsScanner object, using the Gtk-- wrapper

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <gtk--/main.h>
#include <gtk--/entry.h>

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <list>

#include "NewsScannerView.h"
#include "NewsScanner.h"
#include "MongerList.h"
#include "Exceptions.h"

#ifndef PKGDATADIR
	#define PKGDATADIR "/usr/local/share/PicMonger"
#endif

using SigC::slot;
using namespace std;

NewsScannerView::NewsScannerView()
	: pSubject(0),
 	grid( 5,2,true ), 
	frmGroup("Newsgroup"), boxGroup(), btnAddGrp(), btnDelGrp(),
	frmServer("Server"), boxServer(), btnAddServ(), btnDelServ(),
	lblToScanCap	("Total Posts in Group:"), 	// 'cap' as in 'caption'
	lblScannedCap	("Scanning Progress:"), 
	lblFilesFoundCap	("Decoded files this session: "),
	lblBytesReadCap	("Current working filesize:"), 
	chkHTMLPreview("Create HTML Previews"),
	actionBox( GTK_BUTTONBOX_SPREAD, 10 ),
	btnScan("Scan Newsgroup"), btnAbort("Abort"), 
	newScanner( true )
	
{		
	destroy.connect( slot (this, &NewsScannerView::cbQuit) );

	set_border_width(5);

	create_pixmap_items();
	create_comboboxes();
	create_progress_table();

	// Create Preview checkbox
	chkHTMLPreview.set_active( true );
	grid.attach( chkHTMLPreview, 0,2, 4,5 , GTK_EXPAND, GTK_EXPAND, 20, 5);

	create_action_buttons();

	// statusbar
	pack_end( sbar, false, false, 5 );
	
	Update(pSubject);

	show_all();
}

void NewsScannerView::create_pixmap_items()
{
	// add pixmaps to buttons
	addGrpXpm = new Gtk::Pixmap( PKGDATADIR "/add.xpm" );
	delGrpXpm = new Gtk::Pixmap( PKGDATADIR "/del.xpm" );
	addServXpm = new Gtk::Pixmap( PKGDATADIR "/add.xpm" );
	delServXpm = new Gtk::Pixmap( PKGDATADIR "/del.xpm" );
	
	btnAddGrp.add( *addGrpXpm );
	btnAddServ.add( *addServXpm );
	btnDelGrp.add( *delGrpXpm );
	btnDelServ.add( *delServXpm );

	// connect the add/del group/server buttons to their callbacks
	btnAddGrp.clicked.connect( slot(this, &NewsScannerView::cbAddGroup));
	btnDelGrp.clicked.connect( slot(this, &NewsScannerView::cbDelGroup));
	btnAddServ.clicked.connect( slot(this, &NewsScannerView::cbAddServer));
	btnDelServ.clicked.connect( slot (this, &NewsScannerView::cbDelServer));
	
}

void NewsScannerView::create_comboboxes()
{
	// figure out user's home directory
	char *homedir = getenv("HOME");
	if ( homedir == 0 )
	{
		cerr << "HOME environment variable not found; cannot find group/server list."
			<< endl;
		exit(1);
	}

	// groups.mls and servers.mls are kept in $HOME/.picmonger/<file>
	string groupListPath = homedir;
	groupListPath += "/.picmonger/groups.mls";
	string serverListPath = homedir;
	serverListPath += "/.picmonger/servers.mls";

	// create a combobox for servers
	srvs = new MongerList(serverListPath);
	ml_to_gl( srvs, servList );

	cmbServer.set_popdown_strings( servList );
	cmbServer.set_value_in_list( false, false );	// see below
	
	frmServer.set_label_align( 0.5, 0.5 );
	frmServer.set_shadow_type( GTK_SHADOW_ETCHED_IN );
	frmServer.add( boxServer );	

	boxServer.set_border_width(5);
	boxServer.pack_start( cmbServer, true, true, 5 );
	boxServer.pack_start( btnAddServ, false, false, 5);
	boxServer.pack_start( btnDelServ, false, false, 5);
	
	pack_start( frmServer, true, false, 0 );


	// create a combobox for groups
	grps = new MongerList(groupListPath);
	ml_to_gl( grps, groupList );
	
	cmbGroup.set_popdown_strings( groupList );
	cmbGroup.set_value_in_list( false, false ); 
	frmGroup.set_label_align(0.5, 0.5);
	frmGroup.set_shadow_type( GTK_SHADOW_ETCHED_IN );
	frmGroup.add( boxGroup );

	boxGroup.set_border_width(5);
	boxGroup.pack_start( cmbGroup, true, true, 5 );
	boxGroup.pack_start( btnAddGrp, false, false, 5 );
	boxGroup.pack_start( btnDelGrp, false, false, 5 );
	
	pack_start( frmGroup, true, false, 0 );
}

void NewsScannerView::create_progress_table()
{
	pack_start( grid, true, true, 5 );	

	// available post row
	grid.attach( lblToScanCap, 0,1,0,1 );
	grid.attach( entToScanVal, 1,2,0,1 );
	
	entToScanVal.set_editable( false );

	// already scanned (progress) row
	grid.attach( lblScannedCap, 0,1,1,2 );
	grid.attach( prbScannedVal, 1,2,1,2 );
	
	prbScannedVal.set_bar_style( GTK_PROGRESS_CONTINUOUS );
	prbScannedVal.set_orientation( GTK_PROGRESS_LEFT_TO_RIGHT );
	prbScannedVal.set_format_string("%p%%");
	prbScannedVal.set_show_text(true);

	// data files found row
	grid.attach( lblFilesFoundCap, 0,1,2,3 );
	grid.attach( entFilesFoundVal, 1,2,2,3 );
	entFilesFoundVal.set_editable( false );

	// current bytes read row
	grid.attach( lblBytesReadCap, 0,1,3,4 );
	grid.attach( entBytesReadVal, 1,2,3,4 );
	entBytesReadVal.set_editable( false );

}

void NewsScannerView::create_action_buttons()
{
	pack_start( actionBox, false, false, 0 );
	
	// scan button
	actionBox.pack_start( btnScan, false, false, 0 );
	btnScan.clicked.connect( slot( this, &NewsScannerView::cbScan ));

	// abort button
	actionBox.pack_start( btnAbort, false, false, 0 );
	btnAbort.clicked.connect( slot( this, &NewsScannerView::cbAbort ));
	btnAbort.set_state(GTK_STATE_INSENSITIVE);
	btnAbort.set_sensitive( false );
}

NewsScannerView::~NewsScannerView()
{
	killScanner();
	
	delete srvs;
	delete grps;

	delete addGrpXpm;
	delete delGrpXpm;
	
	delete addServXpm;
	delete delServXpm;

}

void NewsScannerView::Update( Subject* callingObject )
{
	static float oldPct = 0.001;	// keeps running count of number of posts scanned
	static int oldDataCount = -1;
	static bool gotTotalPosts = false;
	
	if ( (callingObject == pSubject) && (pSubject != 0) )	
	// don't do this if no scanner
	{
		float scanned = pSubject->pct_scanned();
		unsigned long bread = pSubject->bytes_so_far();

	
		if ( newScanner )	// during first update/setup only
		{
			gotTotalPosts = false;
			entToScanVal.set_text( "Reading..." );
			oldDataCount = -1;
			oldPct = 0.001;
			
			prbScannedVal.configure( 0.0, 0.0, 100.0 );	
			newScanner = false;
		}
		else 	// on all subsequent updates only
		{
			if (!gotTotalPosts)		// only update if we haven't set this field yet
			{
				int posts_in_group = pSubject->num_to_scan();
				if (posts_in_group != 0)
				{
					snprintf( post_total, 6, "%d", posts_in_group );
					entToScanVal.set_text( post_total );
					gotTotalPosts = true;
				}
			}
			
			if (scanned != oldPct) // only update if percentage has changed
			{
				prbScannedVal.set_percentage(scanned);
				oldPct = scanned;
			}
		}
		
		// the rest on first AND all subsequent updates...
		
		int found = pSubject->num_data_files(); 	// update # of data files found
		if (found > oldDataCount) 
		{	// only update if # of found data files changes
			snprintf( pics_found, 6, "%d", found );
			entFilesFoundVal.set_text( pics_found );
			
			oldDataCount = found;
		}

		snprintf( bytes_read, 9, "%ld", bread );	// update bytes read this file
		entBytesReadVal.set_text( bytes_read );
		
		int cid = sbar.get_context_id("update");	// update statusbar
		sbar.pop( cid );
		sbar.push( cid, (pSubject->status()).c_str() );

		// finish all the GUI updates before returning control to the subject
		while ( (Gtk::Main::instance()->events_pending()) != 0 )
		{
			Gtk::Main::instance()->iteration();
		}

	}
}

void NewsScannerView::Warn_Detach(Subject *callingObject)
// This is called when a subject is dying, so we aren't left with a dangling pointer to it
{
	if ( callingObject == pSubject )
	{
		pSubject = 0;
		btnScan.set_state( GTK_STATE_NORMAL );
		btnScan.set_sensitive( true );
	}
}

void NewsScannerView::killScanner()
{
	if (pSubject)
	{
		//pSubject->Detach(this);
		delete pSubject;
		pSubject = 0;
	}
	
	btnScan.set_state(GTK_STATE_NORMAL);
	btnScan.set_sensitive( true );
	btnAbort.set_state(GTK_STATE_INSENSITIVE);
	btnAbort.set_sensitive( false );

	int cid = sbar.get_context_id("killscanner");	// update statusbar
	sbar.pop( cid );
	sbar.push( cid, "Scan completed." );
}

void NewsScannerView::cbScan()
{
	btnAbort.set_state(GTK_STATE_NORMAL);
	btnAbort.set_sensitive( true );
	btnScan.set_state(GTK_STATE_INSENSITIVE);
	btnScan.set_sensitive( false );

	// create a scanner pointing at the selected server/group 
	pSubject = new NewsScanner( 	this, get_server_entry().c_str(), 
						get_group_entry().c_str() );
						
	newScanner = true;	// so Update() knows we're setting up the
						// first time for a new scan when it's called
	
	// let it know to notify this GUI whenever a status update is available
	try {
		pSubject->Attach(this);
	} catch (exc::NullPointer& e) {
		e.handle();
	}
	
	// start scanning the newsgroup for data files
	try {
		pSubject->scan();
	}
	catch (exc::ConnectFail& e) {
		int cid = sbar.get_context_id("error");	// update statusbar
		sbar.pop( cid );
		sbar.push( cid, "Failed to connect to selected server!" );
		e.handle();
		killScanner();
	}
	catch (exc::ResolveFail& e) {
		int cid = sbar.get_context_id("error");	// update statusbar
		sbar.pop( cid );
		sbar.push( cid, "Unable to resolve IP address of server" );
		e.handle();	
		killScanner();
	}
	catch (exc::ReadFail& e) {
		int cid = sbar.get_context_id("error");		// update statusbar
		sbar.pop( cid );
		sbar.push( cid, "Error reading from socket" );
		e.handle();	
		killScanner();
	}
	catch (exc::WriteFail& e) {
		int cid = sbar.get_context_id("error");		// update statusbar
		sbar.pop( cid );
		sbar.push( cid, "Error writing to socket" );
		e.handle();	
		killScanner();
	}
	catch (exc::NNTP_Error& e) {
		// statusbar already handled by NewsScaner::is_NNTP_error()
		e.handle();
		killScanner();
	}
	catch (exc::TimeOut& e) {
		int cid = sbar.get_context_id("error");		// update statusbar
		sbar.pop( cid );
		sbar.push( cid, "Connection to server timed out" );
		e.handle();	
		killScanner();
	}
	
	killScanner();
}


void NewsScannerView::cbAbort()
{
	btnAbort.set_state(GTK_STATE_INSENSITIVE);
	btnAbort.set_sensitive( false );
	
	pSubject->shutdown();
	// Causes the NewsScanner to finish retrieving its current message, 
	// send "QUIT" to the server, finish writing any html preview in progress,
	// and return from NewsScanner::scan() with value "false".
}

void NewsScannerView::cbQuit()
{
	killScanner();
	exit(0);
}

void NewsScannerView::cbAddGroup()
// adds the group currently in cmbGroup entry to the groups.mls file and group dropdown
{
	string ng = get_group_entry();
	grps->add_item( ng );
	groupList.push_back( ng );
	cmbGroup.set_popdown_strings( groupList );
}

void NewsScannerView::cbDelGroup()
// reverses the action of cbAddGroup
{
	string ng = get_group_entry();
	grps->remove_item( ng );

	groupList.clear();
	ml_to_gl( grps, groupList );
	cmbGroup.set_popdown_strings( groupList );
}

void NewsScannerView::cbAddServer()
// adds the server currently in cmbServer entry to the servers.mls file and server dropdown
{
	string ns = get_server_entry();
	srvs->add_item( ns );
	servList.push_back( ns );
	cmbServer.set_popdown_strings( servList );
}

void NewsScannerView::cbDelServer()
// reverses the action of cbAddServer
{
	string ns = get_server_entry();
	srvs->remove_item( ns );

	servList.clear();
	ml_to_gl( srvs, servList );
	cmbServer.set_popdown_strings( servList );
}

string NewsScannerView::get_group_entry()
// returns a string containing the selected text in the group combobox
{
	const Gtk::Entry* grpEntry = cmbGroup.get_entry();
	return grpEntry->get_text();
}

string NewsScannerView::get_server_entry()
// returns a string containing the selected text in the server combobox
{
	const Gtk::Entry* srvEntry = cmbServer.get_entry();
	return srvEntry->get_text();
}

void NewsScannerView::ml_to_gl( MongerList* ml, list<string>& stl_list )
// copies the contents of a MongerList into a STL list<string>
{	
	if (ml->length() > 0)
	{
		ml->reset();
		do
		{
			stl_list.push_back( ml->item() );
		}
		while ( ml->next() );
	}
	else
	{
		stl_list.push_back("");
	}
	stl_list.sort();
}

//void NewsScannerView::set_types( FileTypes ft, bool allow )
//{
//	allowType[ft] = allow;
//}

//bool NewsScannerView::isAllowed( FileTypes ft )
//{
//	return allowType[ft];
//}

bool NewsScannerView::doPreview()
{
	return ( chkHTMLPreview.get_active() );
}


