// NewsScannerView.h
//
// A NewsScannerView is an Observer that displays the current state of the NewsScanner
// instance to which it has subscribed.

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef _NEWS_SCANNER_VIEW_H
#define _NEWS_SCANNER_VIEW_H

#include <gtk--/frame.h>
#include <gtk--/box.h>
#include <gtk--/table.h>
#include <gtk--/combo.h>
#include <gtk--/button.h>
#include <gtk--/label.h>
#include <gtk--/checkbutton.h>
#include <gtk--/buttonbox.h>
#include <gtk--/statusbar.h>
#include <gtk--/pixmap.h>
#include <gtk--/progressbar.h>

#include <list>

#include "Observer.h"
#include "NewsScanner.h"
#include "Exceptions.h"

using namespace std;

class FileTypeDialog;
class MongerList;
class NewsScannerView : public Gtk::VBox, public Observer 
{
	public:
		NewsScannerView();
		virtual ~NewsScannerView();

		virtual void Update(Subject *);
		virtual void Warn_Detach(Subject *);

		//void set_types( FileTypes ft, bool allow );
		//bool isAllowed( FileTypes ft );
		bool doPreview();
		
	private:
		// helper objects
		
		NewsScanner* pSubject;
		MongerList*	 srvs;
		MongerList*  grps;

		// internal data
		list<string> groupList;
		list<string> servList;
		//bool allowType[6];
		char post_total[6];
		char post_done[6];
		char pics_found[6];
		char bytes_read[9];

		// GUI widgets and containers
		Gtk::Table grid;

		Gtk::Frame frmLogo;

		// Group Frame
		Gtk::Frame frmGroup;
		Gtk::HBox  boxGroup;
		Gtk::Combo cmbGroup;
		Gtk::Button btnAddGrp;		void cbAddGroup();
		Gtk::Button btnDelGrp;		void cbDelGroup();

		// Server Frame
		Gtk::Frame frmServer;
		Gtk::HBox  boxServer;
		Gtk::Combo cmbServer;
		Gtk::Button btnAddServ;		void cbAddServer();
		Gtk::Button btnDelServ;		void cbDelServer();

		// Central Table
		Gtk::Label lblToScanCap;		Gtk::Entry entToScanVal;
		Gtk::Label lblScannedCap;	Gtk::ProgressBar prbScannedVal;
		Gtk::Label lblFilesFoundCap;	Gtk::Entry entFilesFoundVal;
		Gtk::Label lblBytesReadCap;	Gtk::Entry entBytesReadVal;
		Gtk::CheckButton	chkHTMLPreview;

		// Action Buttons
		Gtk::HButtonBox	actionBox;
		Gtk::Button btnScan;		void cbScan();
		Gtk::Button btnAbort;		void cbAbort();
		
		Gtk::Statusbar sbar;

		// pixmaps
		Gtk::Pixmap *addGrpXpm;
		Gtk::Pixmap *delGrpXpm;
		Gtk::Pixmap *addServXpm;
		Gtk::Pixmap *delServXpm;

		void cbQuit();

		// miscellaneous
		string get_group_entry();
		string get_server_entry();
		void ml_to_gl( MongerList* ml, list<string>& stl_list);
		void killScanner();
		bool newScanner;

		// setup methods
		void create_pixmap_items();
		void create_comboboxes();
		void create_progress_table();
		void create_action_buttons();
};

#endif
