
// OptionsTab.cpp
// Implementation of the general/miscellaneous options tab GUI

#include "OptionsTab.h"
#include <stdlib.h>		// need getenv()
#include <unistd.h>		// need chdir()
#include <iostream>

using std::cerr;

OptionsTab::OptionsTab()
	: Gtk::Table( 4, 1, true )
{
	set_border_width( 5 );
	set_row_spacings( 5 );

	// create and connect all the widgets
	frmPath = manage(new Gtk::Frame("Decoded File Path"));
	frmPath->set_border_width(5);
	frmPath->set_label_align( 0.5 );
	
	frmHtmlGen = manage( HtmlOptions::Instance() );

	tblPath = manage(new Gtk::Table(2,5,true));

	// get current working dir to fill entry box on creation
	char pwdbuf[128];
	getcwd( pwdbuf, 128 );
	string pwd(pwdbuf);
	
	entPath = manage(new Gtk::Entry());
	entPath->set_editable( true );
	entPath->set_text( pwd );
	entPath->select_region(0, entPath->get_text_length());
	entPath->activate.connect( slot( this, &OptionsTab::cbTextChange ));
		
	btnBrowse = manage(new Gtk::Button("Browse..."));
	btnBrowse->clicked.connect( slot( this, &OptionsTab::cbBrowse ));

	// pack the widgets into the tab
	// path setting area
	tblPath->attach( *entPath, 0,5,0,1, (GTK_FILL|GTK_EXPAND), GTK_EXPAND );
	tblPath->attach( *btnBrowse, 4,5,1,2, GTK_EXPAND, GTK_EXPAND );
	frmPath->add( *tblPath );
	attach( *frmPath, 0,1,0,1 );

	// HtmlGenerator Options (not yet implemented)
	attach( *frmHtmlGen, 0,1,1,4 );

	// set up file selection dialog
	fileSel = manage( new Gtk::FileSelection("Change Working Directory") );
	fileSel->set_filename( pwd );
	fileSel->show_fileop_buttons();
	(fileSel->get_ok_button())->
		clicked.connect( slot( this, &OptionsTab::cbFileSelOK ));
	(fileSel->get_cancel_button())->
		clicked.connect( slot( this, &OptionsTab::cbFileSelCancel ));
	fileSel->hide();

	show_all();
}

OptionsTab::~OptionsTab() {}

void OptionsTab::cbBrowse()
{
	fileSel->show();	
}

void OptionsTab::cbFileSelOK()
{
	// use the dialog to fill in the entry box and manually call the entry box
	// callback to change directories (avoid code duplication)
	entPath->set_text( fileSel->get_filename() );
	cbTextChange();
	
	fileSel->hide();

	// this looks useless, but it sets the dialog back to a state where the
	// path is current and the entry box is empty
	fileSel->set_filename( fileSel->get_filename() );
}

void OptionsTab::cbFileSelCancel()
{
	// hide file selection dialog and make sure it's still consistent with the
	// current directory 
	fileSel->hide();
	fileSel->set_filename( entPath->get_text() );
}

void OptionsTab::cbTextChange()
{
	int result = chdir( (entPath->get_text()).c_str() );
	if ( result == 0 )
		return;
	else	// invalid directory selected
	{
		cerr 	<< "\a\n"
				<< "Path NOT Changed: Directory selected is invalid or not accessible"
				<< endl;

		// reset text entry so it's consistent with current dir
		char pwdbuf[128];
		getcwd( pwdbuf, 128 );
		string pwd(pwdbuf);
		entPath->set_text( pwd );
	}
}