// PicMonger.cpp
// 
//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "PicMonger.h"

#ifndef PACKAGE
	#define PACKAGE "PicMonger"
#endif

#ifndef VERSION
	#define VERSION "devel"
#endif

#ifndef PKGDATADIR
	#define PKGDATADIR "/usr/local/share/PicMonger"
#endif

PicMonger::PicMonger()
	: Gtk::Window( GTK_WINDOW_TOPLEVEL ), boxMain(false, 5)
{
	set_border_width( 10 );
	add(boxMain);

	string title(PACKAGE);
	title += " ";
	title += VERSION;
	set_title( title );
	
	// create PicMonger logo at top
	logo = manage( new Gtk::Pixmap( PKGDATADIR "/pmlogo.xpm") );
	frmLogo.add( *logo );
	frmLogo.set_shadow_type( GTK_SHADOW_OUT );
	boxMain.pack_start(frmLogo);
	
	// create tabbed notebook at bottom
	p_notebook = manage( new Gtk::Notebook() );
	p_notebook->set_tab_pos( GTK_POS_TOP );
	boxMain.pack_end( *p_notebook );

	// add main scanner view tab to notebook
	p_nsv = manage( new NewsScannerView() );
	p_notebook->pages().push_back( Gtk::Notebook_Helpers::TabElem( *p_nsv, "Scanner" ));

	// add filetypes tab to notebook
	p_filetypes = manage( new FileTypeDialog() );
	p_notebook->pages().push_back(
			Gtk::Notebook_Helpers::TabElem( *p_filetypes, "File Types" ) );

	// add options tab to notebook
	p_options = manage( new OptionsTab() );
	p_notebook->pages().push_back(
			Gtk::Notebook_Helpers::TabElem( *p_options, "Options" ));

	// add authorization tab to notebook
	p_auth = manage( AuthTab::Instance() );
	p_notebook->pages().push_back(
			Gtk::Notebook_Helpers::TabElem( *p_auth, "Auth" ));
			
	// add about tab to notebook
	p_about = manage( new AboutTab() );
	p_notebook->pages().push_back( Gtk::Notebook_Helpers::TabElem( *p_about, "About" ));
			
	show_all();

}

PicMonger::~PicMonger()
{

}