// PreviewPage.cpp
// Represents one page of the HTML preview set

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "PreviewPage.h"
#include "Thumbnail.h"
#include "HtmlOptionState.h"

#include <algorithm>

using namespace std;

// ctor
PreviewPage::PreviewPage(	const string& path,
							const string& _filename,
							const string& group		)
							
	: HtmlPage( path, _filename, group ), rows( 3 ), cols( 4 ),
	showFilenames( true ), showDimensions( true )
{
	// set height & width constraints for thumbnail size
	t_height = HtmlOptionState::Instance()->get_t_height();
	t_width = HtmlOptionState::Instance()->get_t_width();
}

//dtor
PreviewPage::~PreviewPage()
{}

void 
PreviewPage::add_item( const string& itemFile )
// add an image to the preview page
{
	if (!is_full())
	{
		imageFiles.push_back( itemFile );
	}
}

void 
PreviewPage::add_link( const LinkType& link )
// add a link (usually navigational) to be included at the bottom of the page
{
	links.push_back( link );
}

void 
PreviewPage::write_content(ofstream& fout)
{
	uint &img_per_row = cols;
	uint &row_per_page = rows;
	
	fout << "<center>" << endl << "<table cellspacing=10>" << endl;

	sort( imageFiles.begin(), imageFiles.end() );
	
	vector<string>::iterator iter = imageFiles.begin();

	uint col = 0;
	uint row = 0;
	uint tablearea = img_per_row * row_per_page;

	Thumbnail *thumb;
	
	DimensionType constraint;
	constraint.height = t_height;
	constraint.width = t_width;
	
	for (uint i=0; i < tablearea ; i++)
	{
		if (col == 0) 
		{
			fout << "<tr>" << endl;
			col++;
		}

		// write cells
		if ( iter != imageFiles.end() ) iter++;
		if ( (row == 0) && (col == 1) ) iter = imageFiles.begin();

		if ( iter != imageFiles.end() )
		{	
			// create a thumbnail of the current image
			thumb = new Thumbnail( *iter, constraint );
			
			// write a filled cell
			fout << "<td>" << endl;

			fout 	<< "<center><a href=\"" << (*iter) << "\"> "
				 	<< "<img src=\"" << (*iter) << "\" border=2 "
				 	<< "height = " << thumb->get_height() << " "
				 	<< "width = " << thumb->get_width() << " "
				 	<< ">"
				 	<< "</a></center>" << endl;

			if (showFilenames)
				fout << "<center>" << (*iter) << "</center>" << endl;
				
			if (showDimensions)
			{
				fout 	<< "<center>" << thumb->get_full_width() << "x" 
						<< thumb->get_full_height() << "</center>" << endl;
			}
					
			fout << "</td>" << endl << endl;
			delete thumb;
		}
		else
		{	// write an empty cell
			fout << "<td></td>" << endl;
		}

		col++;
		
		if (col == (img_per_row + 1))
		{
			fout << "</tr>" << endl;
			col = 0;
			row++;
		}
	}
	
	fout << "</table>" << endl << "</center>" << endl;
	fout << endl;
	
	fout << "<hr width=\"90%\">" << endl;

	if (!links.empty())
	// write navigation links at the bottom of the page
	{
		fout << "<center>\n";
		
		vector<LinkType>::iterator linkIter = links.begin();
		while (linkIter != links.end())
		{
			fout << "\t"
				<< "<a href = \"" << linkIter->url << "\">" << linkIter->description
				<< "</a>" << endl;
			linkIter++;
		}	

		fout << "</center>" << endl;
	}
}

