// PreviewPage.h
// 
// PreviewPage is a subclass of HtmlPage that represents a one page of image previews.

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _PREVIEWPAGE_H
#define _PREVIEWPAGE_H

#include "HtmlPage.h"
#include <string>
#include <vector>
#include <fstream>

struct LinkType {
	string url;
	string description;
};

using namespace std;
class PreviewPage : public HtmlPage {

	public:
		PreviewPage(	const string& path,
						const string& _filename,
						const string& group	);
		~PreviewPage();

		void add_item(const string& imagefile);
		void add_link(const LinkType& link);

		inline void set_rows(const uint r)
			{ rows = r;	}
		inline void set_cols(const uint c)
			{ cols = c;	}
		inline void set_thumb_size( const uint h, const uint w )
			{
				t_height = h;
				t_width = w;
			}
		inline void set_show_filenames(const bool show)	
			{ showFilenames = show;	}
		inline void set_show_dimensions(const bool show)
			{ showDimensions = show; }
		inline bool is_full()
			{ return (imageFiles.size() == ( rows * cols ));	}
		
	protected:
		void write_content(ofstream& ofs);
		
	private:
		uint rows, cols, t_height, t_width;
		bool showFilenames;
		bool showDimensions;
		
		vector<string> imageFiles;
		vector<LinkType> links;

};

#endif