// Subject.C
// 
// Bare implementation of Subject class, inherited by all objects to which Observers can subscribe

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "Subject.h"
#include "Observer.h"
#include "Exceptions.h"
#include <list>

using namespace std;

Subject::Subject() 
	: _observers(), iObs(_observers.begin())
{}

Subject::~Subject() 
{
	// Iterate through the observers and detach them before destruction

	if (!(_observers.empty()))
	{
		iObs = _observers.begin();
		while ( iObs != _observers.end() )
		{
			Detach(*iObs);
			iObs++;
		}

		_observers.clear();
	}
}

void Subject::Attach(Observer* obs) throw (exc::NullPointer)
{
	//	Add an observer to the notify list
	if (obs)
		_observers.push_back(obs);
	else
		throw exc::NullPointer( "Subject::Attach()" );
}

void Subject::Detach(Observer* obs) throw (exc::NullPointer)
{
	//obs->Warn_Detach(this);
	if (!_observers.empty())
	{
		if (obs) 
			_observers.remove(obs);
		else
			throw exc::NullPointer( "Subject::Detach()" );			
	}
}

void Subject::Notify() throw (exc::NullPointer)
{
	// 	for each subscribed observer
	// 		signal it to update
	if (!_observers.empty())		// don't bother if observer list is empty
	{
		iObs = _observers.begin();
		while (iObs != _observers.end())
		{
			if (*iObs)
			{
				(*iObs)->Update(this);
				iObs++;
			}
			else
				throw exc::NullPointer( "Subject::Notify()" );
		}
	}
}




