// Thumbnail.cpp
// 
//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "Thumbnail.h"
#include <iostream>
using std::cout;

Thumbnail::Thumbnail( const string& imgpath, const DimensionType& scale_to )
	: constraint( scale_to )
{	
	image.load( imgpath );
	height = image.rgb_height();
	width = image.rgb_width();
	
	int dh = height - constraint.height;
	int dw = width - constraint.width;
	
	bool already_fits = ( (dh < 0) && (dw < 0) );
	
	if ( already_fits )		// if this is true, the constraint is bigger than the
	{						// image in both dimensions, so there's no need to scale
		scaled.height = height;
		scaled.width = width;
	}
	else				// the image dimensions exceed the constraint dimensions by
	{					// a maximum of dh (or dw) lines
		float scale_h = static_cast<float>(constraint.height) / 
						static_cast<float>(height);
		float scale_w = static_cast<float>(constraint.width) / 
						static_cast<float>(width);

		// find the maximum scaling factor that will keep both dimensions within the
		// constraint dimensions
		float scale_all;
		if ( (scale_h > 0.0) && (scale_w > 0.0) )
			scale_all = ( scale_h < scale_w ) ? scale_h : scale_w;
		else if ( scale_h < 0.0 )
			scale_all = scale_w;
		else
			scale_all = scale_h;

		// ... and calculate the scaled dimensions
		scaled.height = static_cast<int>( static_cast<float>(height) * scale_all );
		scaled.width = static_cast<int>( static_cast<float>(width) * scale_all );
	}
}

