// Bootstrap/Main for PicMonger
// picsuck.cpp

//	PicMonger automates retrieval, decoding, and previewing of Usenet newsgroup files.
//	Copyright (C) 1999,2000		David J.V. Brown, MD
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "ImlibSingleton.h"
#include "PicMonger.h"
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <gtk--/main.h>
#include <exception>

using namespace std;

bool check_config_file( const string& path );

int main(int argc, char *argv[])
{	
	char *homedir = getenv("HOME");
	if (!homedir)
	{
		cerr << "\a\nHOME environment variable not found.\n";
		cerr << "Picmonger uses this variable to find your configuration files.\n";
		cerr << "Aborting..." << endl;
		return 1;
	}

	string pmpath = homedir;
	pmpath += "/.picmonger/";

	fstream pathcheck;

	// check for ~/.picmonger and try to create it if it doesn't exist...
	pathcheck.open( pmpath.c_str(), ios::in );
	
	if ( !pathcheck.is_open() )
	// couldn't open it... try creating it
	{
		string mkdir_cmd = "mkdir ";
		mkdir_cmd += pmpath;

		int mkdir_result = system( mkdir_cmd.c_str() );
		
		if ( mkdir_result != 0 )
		{
		cerr << "\aCouldn't create picmonger config path: " << pmpath << endl;
		cerr << "Aborting..." << endl;
		return 1;
		}
	}
	else
	{
		pathcheck.close();
	}
	
	bool servers_ok = check_config_file( pmpath+"servers.mls" );
	bool groups_ok = check_config_file( pmpath+"groups.mls" );

	if (servers_ok && groups_ok)
	{	
		// config files operational... run the app
		Gtk::Main gmain( &argc, &argv);		

		// create imlib once for the whole run of the app
		ImlibSingleton *p_imlib = ImlibSingleton::Instance();

		PicMonger picmonger;
		gmain.run();

		// destroy imlib before exiting
		delete p_imlib;
		return 0;
	}
	else
	{
		// config files inaccessible... abort!
		cerr << "\a\nConfig file neither found nor created.  Aborting..." << endl;
		return 1;
	}
}
		
bool check_config_file( const string& path )
{
	fstream mls_file;

	// check for servers.mls and try to create if not there;
	mls_file.open( path.c_str(), ios::in );
	
	if ( !mls_file.is_open() )
	{
		// not found or access denied
		mls_file.open( path.c_str(), ios::out );
		if ( !mls_file.is_open() )
		{
			// can't open path for write, either...
			cerr << "\aCould not create config file in " << path << endl;
			cerr << "Make sure the path exists and that you have read/write ";
			cerr << "permissions." << endl;
			return false;
		}
		else
		{
			// write ok, so create empty config file
			mls_file << "";
		}
	}

	mls_file.close();
	return true;
}

